/*! *************************************************************************************************
 * @file   SPI_Driver.h
 *
 * @author Thomas Morey <Thomas.Morey1@navy.mil> ESDP,
 * @       Alfredo Sepulveda <Alfredo.Sepulveda@navy.mil>
 *
 * @date   July 30, 2019
 *
 * @brief  header file to initialize SPI for C66
 ******************************************************************************************************/

/*! *************************************************************************************************
 * GPIO Layout for CS2
 *
 *
 ******************************************************************************************************/

#ifndef SPI_DRIVER_H
#define SPI_DRIVER_H

#include <stdio.h>
#include <stdint.h>
#include <string.h>

/*! Driver Header file*/
#include <ti/csl/csl.h>
#include <ti/csl/hw_types.h>
#include <ti/drv/spi/SPI.h>
#include <ti/drv/spi/src/v0/SPI_v0.h>
#include <ti/drv/spi/soc/SPI_soc.h>
#include <ti/drv/spi/test/src/SPI_log.h>
#include "SPI_Driver.h"

/*! ******************************************************************************
 * @section  Structures
 *********************************************************************************/
typedef struct
{
    char mode;                  //m=master, s=slave
    uint32_t bitrate;           // in Hz
    uint8_t transferSize;       // SPI data frame size in bit
    uint8_t pinMode;            // 3 pin or 4 pin mode
    uint8_t chipSelect;         // cs0 thru cs3 can be selected
    uint32_t systemClock;       // System clock in Hz
} spiuserParams;

// Handle to open SPI connection
typedef void* SPI_Hand;

/*! ******************************************************************************
 * @section  Global Variables
 *********************************************************************************/
#define enable 1        /*! used in SPI */
#define disable 0       /*! which SPI to initialize */


uint8_t  rxBuf[100];
uint8_t  txBuf[100];
uint16_t dataBuffer;
uint16_t VAR_TempCounter;
// TI Library Variables
SPI_Params spiParams; /*! SPI Parameter structure */
SPI_v0_HWAttrs spi_cfg;

/********************************************************************************
 *  @fn          SPI_DriverInit();
 *  @brief       Initialize SPI on device
 *
 *  @pre         none
 *
 *  @param[in]   param          User parameter struct for peripheral configuration values
 *  @param[out]  hSPI           Handle to SPI instance
 *  @param[in]   peripheralNum  Which SPI peripheral to initialize
 *
 *  @return      hSPI
 *
 *  @sa          none
 *********************************************************************************/
void SPI_DriverInit(spiuserParams param, SPI_Hand *hSPI, uint8_t peripheralNum);

//! Not used, for future use
/*****************************************************************************************
 *   @fn transferCallback()
 *   @brief transferCallback is used when set to callback as a SWI.
 *          user can set flags within the callback.
 *
 *   @return none
 *****************************************************************************************/
void transferCallback();

/*!****************************************************************************************
 *   @fn     SPI2_transaction_blocking();
 *   @brief  read and write transaction for SPI, blocks while transaction is in process
 *
 *   @param[in]   hSPI    handle to SPI driver instance
 *   @param[in]   length  number of frames in a frame
 *   @param[in]   txBuf   data to write on the bus
 *   @param[out]  rxBuf   pointer to receive buffer
 *
 *   @return spi2_transaction.status
 *           -COMPLETED  = 0
 *           -STARTED    = 1
 *           -CANCELED   = 2
 *           -FAILED     = 3
 *           -DEASSERT   = 4
*****************************************************************************************/
uint16_t SPI2_transaction_blocking(SPI_Hand hSPI, uint8_t length, void *txBuf, void *rxBuf);

void SPI_SendReceive(uint8_t *, uint8_t *, uint16_t , bool );

void ConfigureUserParams(spiuserParams *hSPI);
;
#endif
