/*! *************************************************************************************************
 * @file   SPI_Driver
 *
 * @author Thomas Morey <Thomas.Morey1@navy.mil> ESDP
 *
 * @date   18 October 2018
 *
 * @brief  Initialize SPI for C66
 *         Hard Coded to initialize SPI2 via: #define peripheralNum     2
 *         Considerations to use would per specific applications would be
 *         -transmit and receive buffer sizes
 *         -SPI_Params for application such as frameFormat
 *         -spi_cfg for interrupts, loopback or DMA usage
 *
 * @pre      none
 * @return   none
 * @sa       none
 *
 ******************************************************************************************************/

/*! ***********************************************************
 * @Section  Header Files
 *************************************************************/
/*! Driver Header file*/
#include <c6x.h>
#include <ti/csl/csl.h>
#include <ti/csl/hw_types.h>
#include <ti/drv/spi/SPI.h>
#include <ti/drv/spi/src/v0/SPI_v0.h>
#include <ti/drv/spi/soc/SPI_soc.h>
#include <ti/drv/spi/test/src/SPI_log.h>
#include "SPI_Driver.h"


#define LSB 0x0010000
#define MSB 0x0000000
#define NUM_SPI_PORTS 2

void CycleDelay(unsigned int count);




SPI_Hand hSPI;
spiuserParams userConfig; // There's 2 separate configurations required,
                          // one for the user configuration, and another for the board
                          // that one is configured in the SPI_DriverInit function


/********************************************************************************
 *  @fn          Main();
 *  @brief       Test SPI transactions
 *  @pre         none
 *  @return      none
 *  @sa          none
 *********************************************************************************/
void main(void)
{
    TSCL = 0;

    // The driver is initialized for use with SPI2. There are 3 onboard SPI modules
    ConfigureUserParams(&userConfig);
    CycleDelay(100000);
    SPI_DriverInit(userConfig, &hSPI,  0);

    txBuf[0] = 0xA; // put some data in the tx buffer to send to SPI Slave Device
    txBuf[1] = 0xB;
    txBuf[2] = 0xC;
    txBuf[3] = 0xD;
    txBuf[4] = 0xE;
    txBuf[5] = 0xa;

    while(1)
    {


        SPI_SendReceive(txBuf, rxBuf, 6, true);  // Send four 8-bit transactions
        CycleDelay(100000);


        SPI_SendReceive(txBuf, rxBuf, 5, true);  // Send four 8-bit transactions
        CycleDelay(100000);
        SPI_SendReceive(txBuf, rxBuf, 5, false); // Receive four 8-Bit transactions

    }
}

// Configure the driver to work with device here.
// To edit the phase and the polarity See SPI_DriverInit();
void ConfigureUserParams(spiuserParams *hSPI)
{
    hSPI->mode = 'm';                // Master Mode
    hSPI->bitrate = 1e6;             // 1MHz
    hSPI->transferSize = 8;         // SPI bit transfersize
    hSPI->pinMode = 1;              // 3 or 4 pin, 1 = 4pin,
    hSPI->chipSelect = 0;           // Enable CS0, SPI_2 is tied to CS0
    hSPI->systemClock = 1200000000; // 1.2GHz

}



/********************************************************************************
 *  @fn          SPI_DriverInit();
 *  @brief       Initialize SPI on device
 *
 *  @pre         none
 *
 *  @param[in]   param          User parameter struct for peripheral configuration values
 *  @param[out]  hSPI           Handle to SPI instance
 *  @param[in]   peripheralNum  Which SPI peripheral to initialize
 *
 *  @return      none
 *
 *  @sa          none
 *********************************************************************************/
void SPI_DriverInit(spiuserParams param, SPI_Hand *hSPI, uint8_t peripheralNum)
{
    /*!**********************************************
     * Step 1: set spi_cfg configuration structure
    ************************************************/
    /*! Get the default SPI init configurations */
    SPI_socGetInitCfg(peripheralNum, &spi_cfg);

    /*! Update SPI parameters to fit needs*/
    spi_cfg.dmaMode     = FALSE;
    spi_cfg.enableIntr  = FALSE; /*polling mode = false*/
    spi_cfg.pinMode     = param.pinMode;
    spi_cfg.csNum       = param.chipSelect;
    spi_cfg.loopback    = FALSE;
    /* Update the SPI functional clock based on CPU clock*/
    spi_cfg.inputClkFreq = param.systemClock / SPI_MODULE_CLOCK_DIVIDER;

    /*! Set the updated SPI initialization configurations */
    SPI_socSetInitCfg(peripheralNum, &spi_cfg);

    /*!**********************************************
    *  Step 2: set spiParams SPI Parameters
    ************************************************/
    // The spec for SPI transfers is unique for TI
    // you may need to cross reference the device you
    // are using and how they match with TI's implimentation
    spiParams.frameFormat = SPI_POL1_PHA0;   /*!< SPI mode Polarity 0 Phase 0 */
    spiParams.bitRate = param.bitrate;       /*!< SPI bit rate in Hz */
    spiParams.dataSize = param.transferSize; /*!< SPI data frame size in bits */

    /*!*******************************************************************************
         * SPI_MODE_BLOCKING. This mode can only be used when called within a Task context
         * SPI_MODE_CALLBACK. This mode can be used in a Task, Swi, or Hwi context.
         *********************************************************************************/
    // Non - Interrupt based
    spiParams.transferMode = SPI_MODE_BLOCKING;

    if (param.mode == 'm')
    {
        spiParams.mode = SPI_MASTER; /*!< SPI in master mode */
    }
    else if (param.mode == 's')
    {
        spiParams.mode = SPI_SLAVE; /*!< SPI in slave mode */
    }
    else
        printf("mode not set, master=m slave=s");

    /*!**********************************************
    *
    * @fn SPI_open()
    * @brief    Step 3: Open SPI instance
    *
    * @pre      SPI_setParams()
    *
    * @param    peripheralNum  SPI to open
    *           -SPI0 for NAND access
    *           -SPI1
    *           -SPI2
    * @param    &spiParams
    *
    ***********************************************/
    SPI_init();
    *hSPI = (void *)SPI_open(peripheralNum, &spiParams);
    if (!*hSPI)
    {
        printf("\nSPI_Open failed! \n");
    }
    else
        printf("\nSPI_Open Success!. \n");

    /*!**********************************************
    * @brief    Step 4: modify any registers for specifications
    ************************************************/

    /*!**********************************
        * @brief  Change shift direction
        *************************************/
    /*! get address of SPIFMT0 for SPIn */
    // uint32_t SPIFMT0_addr = (spi_cfg.baseAddr | 0x50);

    // /*!read data at address, "OR" with DIR bit */
    // uint32_t SPIFMT0_data = (HW_RD_REG32(SPIFMT0_addr) | MSB);

    // /*!write data to SPIFMT0 for SPIn */
    // HW_WR_REG32(SPIFMT0_addr, SPIFMT0_data);
    uint32_t *ptr = (uint32_t*)(spi_cfg.baseAddr + 0x3c);
    *ptr &= 0xEFFFFFFF;
    /*!**********************************************
    * @brief    Step 5: LAST STEP!!! Enable SPI Global Clock
    ***********************************************/

    int xferEnable = 1;
    SPI_control(*hSPI, SPI_V0_CMD_XFER_ACTIVATE, (void *)&xferEnable);

    int byteOrder = 0; //msb out first
    SPI_control(*hSPI, SPI_V0_CMD_SHIFT_DIR, (void *)&byteOrder);
}

/*****************************************************************************************
 *   @fn     transferCallback()
 *   @brief  transferCallback is used when set to callback as a SWI.
 *           user can set flags within the callback.
 *
 *   @return none
 *****************************************************************************************/
void transferCallback(SPI_Handle handle, SPI_Transaction *transaction)
{
}

/*!****************************************************************************************
 *   @fn     SPI2_transaction_blocking();
 *   @brief  read and write transaction for SPI, blocks while transaction is in process
 *
 *   @param[in]   hSPI    handle to SPI driver instance
 *   @param[in]   length  number of frames in a frame
 *   @param[in]   txBuf   data to write on the bus
 *   @param[out]  rxBuf   pointer to receive buffer
 *
 *   @return spi2_transaction.status
 *           -COMPLETED  = 0
 *           -STARTED    = 1
 *           -CANCELED   = 2
 *           -FAILED     = 3
 *           -DEASSERT   = 4
*****************************************************************************************/
uint16_t SPI2_transaction_blocking(SPI_Hand hSPI, uint8_t length, void *txBuf, void *rxBuf)
{
    SPI_Transaction spi2_transaction;
    // Bug Fix if this argument is left uninitialized the driver may fail
    uint8_t x = 1;
    spi2_transaction.arg = &x;
    // End bug fix
    spi2_transaction.count = length; /*! number of frames per transaction */
    spi2_transaction.txBuf = txBuf;  /*! buffer to be written */
    spi2_transaction.rxBuf = rxBuf;  /*! buffer holding the received data*/
    SPI_transfer(hSPI, &spi2_transaction);
    return spi2_transaction.status;
}



void SPI_SendReceive(uint8_t *tx, uint8_t *rx, uint16_t transactionSize, bool txRx)
{
    // This program sends 3bytes over SPI
    if(txRx == true)
    {
        SPI2_transaction_blocking((SPI_Handle)hSPI, transactionSize, tx, rx);
    }

    else
    {
        SPI2_transaction_blocking((SPI_Handle)hSPI, transactionSize, tx, rx);
        // Data is saved in the RX buffer
        // Read The Rx Buffer Back
        for(VAR_TempCounter = 0; VAR_TempCounter < transactionSize; VAR_TempCounter = VAR_TempCounter +2)
        {
            dataBuffer = rx[VAR_TempCounter] << 8;
            dataBuffer = dataBuffer | rx[VAR_TempCounter + 1] ;

        }
    }
}


void CycleDelay(unsigned int count)
{
    unsigned int TSCLin;

    if(count <= 0)
        return;

    /* Get the current TSCL */
    TSCLin = TSCL;

    while((TSCL - TSCLin) < (uint32_t) count);

}
